package Mailform;

=head1 NAME

Mailform - Create E-mail input form and send E-mail to recipient

$Id: Mailform.pm,v 1.3 2002/11/04 14:16:31 t_shigetome Exp $

=head1 SYNOPSIS

	use CGI;
	use Mailform;
	my(%config) = (
		'subject'	=>	'No subject',
		'redirect'	=>	'mailtest.txt', # It's for debug
		'sendmail'	=>	['/usr/lib/sendmail -n -t -oi'],
		'nexturl'	=>	'index.html',
	);
	my(%recipient) = (
		'Someone'		=>	'someone@anywhere.com',
		'Anotherone'	=>	'Anotherone@anywhere.com',
		'Each'			=>	'Someone;Anotherone',
		'All'			=>	'Someone;Anotherone;boss@anywhere.com',
	);
	my($form) = new Mailform($ENV{'REQUEST_URI'});
	$form->ParseQuery();
	$form->SetParameter(\%config, \%recipient);
	$form->OutResponse();

=head1 DESCRIPTION

This is for CGI program.
Generate E-mail send form.
Posted message will be send to selected receiver.
Support base64 encoding for unsafe character in subject and name.
This form support only Japanese Kanji code like as Shift-JIS.

=head2 CGI ARGUMENT

=over 4

=item config

Value of `config' in QUERY_STRING.
That value is used for hint of selecting from parameter set.

=item nexturl

Use with location header after post form.
Default value is HTTP_REFERER.

=back

=head2 METHOD

=over 4

=item new( $mailform_URI)

Create new instance
The $mailform_URI is used at action attribute in input form.

	my($form) = new Mailform($ENV{'REQUEST_URI'});

=item SetParameter($filename)  or SetParameter($rh_para, $rh_alias)

Setup of parameter.
The $filename is filename of parameter file.
That parameter file will be parsed with F<Params.pm>.

The $rh_para and $rh_alias are hash reference of parameter.
See following description for parameters.
The $rh_alias is hash refrence of recipient list.
If you want to assign two or more receivers to one alias, you can describe separated each one `;'.

=head3 Parameter description

=over 8

=item subject

Specify default subject.

=item nexturl

Specify default next URL after post mail.

=item sendmail

Optional parameter.
Specify sendmail command and options.
For detail, see C<sub SendMail>.

=item redirect

It's for debug.
Do not sendmail, but save to specified file to post message with MIME header.

=item recipient

Specify E-mail recipient as alias.
It can be nested by alias name.

=back

Refer following sample source to use parameter file.

sample.cgi

	use CGI;
	use Mailform;
	my($form) = new Mailform($ENV{'REQUEST_URI'});
	$form->ParseQuery();
	$main::configname = $form->config();
	$form->SetParameter("sample.par");
	$form->OutResponse();

sample.par

	#if $main::configname eq 'test'
	#	define CONFIG	test
	#endif
	%CONFIG% redirect{
		testmail.txt;
	};
	%CONFIG% subject{
		default subject
	};
	%CONFIG%	nexturl{
		/~yahoo/index.html;
	};
	recipient foo{
		foo@anywhere.com;
	};
	recipient ya{
		ya@anytime.com;
	};
	recipient all{
		ya;
		foo;
	};


=item ParseQuery()

Parse query string or post data.

=item OutResponse()

Output result.
If it send mail, it will be return location header and move to next URL.

=item SetTemplate($templatefile)

The $templatefile is template file name.
It is used as template of mailform page.
You can customize that. But it's not simple.
That template file is expanded with replace keys.
For detail, see default template data in C<sub GetTemplate>.
See also C<InputFormTemplateFile>

=item InputFormTemplateFile($filename)

The $filename is template file name.
It is used as input form template.
You can customize that.
That template file is expanded at replace keys.
For detail, see default template data in C<sub GetFormTemplate>.

=item debug($debug_opt)

Toggle each debug condition.
Default is debug off.
There are five debugging options.

	$form->debug('configpara');
	$form->debug('query_string');
	$form->debug('alias');
	$form->debug('template');
	$form->debug('sendmail');

Message of debugging will be outputted to STDERR.
Also. you can log by redirect STDERR with F<debuglog.pl> or other.

When you terminate debugging, you can strip debugging code from this module.
All of debugging code are between  line of `#>>>DEBUG_CODE'.
These are able to strip by F<striper.pl>.

=over 4

=item configpara

Debug of parameter

=item query_string

Debug of parse query string or post data

=item alias

Debug of alias expand

=item template

Debug of template expand

=item sendmail

Debug of sendmail command
Do not execute sendmail command.
Save to file that is specified in parameter as `redirect'.

=back

=back

=head1 SEE ALSO

F<Params.pm>, F<CGI.pm>, F<MIME::Base64.pm>, F<jcode.pl>

=head1 AUTHOR

Tetsuya Shigetome, E<lt>t_shigetome@muf.biglobe.ne.jpE<gt>.

=head1 HISTORY

=over 4

=item $Revision: 1.3 $

$Log: Mailform.pm,v $
Revision 1.3  2002/11/04 14:16:31  t_shigetome
o Change comments


=item Revision: 1.2 

Log: Mailform.pm,v 
Revision 1.2  2002/11/04 13:25:47  t_shigetome

o Change manual to english.

o Change to Params.pm from loadparaex.pl
  And add hash reference arguments to SetParameter.


=back

=cut

$Mailform::VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);

use strict;
use Carp;
use Params;
require 'jcode.pl';
use MIME::Base64;
############################################
# Input Guide messages
# You can find with following key word.
#	'_EMPTY_EMAIL_FILED_'
#	'_INVALID_EMAIL_ADDRESS_'
#	'_EMPTY_SUBJECT_FILED_'
#	'_EMPTY_BODY_FILED_'
#	'_ANY_ERRORS_'
#	'_START_MESSAGE_';
#	'_THANK_YOU_'
#
######################
# Constructor
# To E-mail address, ML members hash ref, Mail form CGI URI
sub new
{
my($pkg,$s_cgi_uri) = @_;
my($rh_this);
# 
	$rh_this =  bless {
		'_debug'				=>	{
			'configpara'			=>	undef,
			'query_string'			=>	undef,
			'alias'					=>	undef,
			'template'				=>	undef,
			'sendmail'				=>	undef,
		},
		'_Config'				=>	{
			'_Redirect'				=>	undef,
			'_Sendmail'				=>	undef,
		},
		'_AnyError'				=>	0,
		'_CgiQuery'				=>	undef,
		'_MailEntity'			=>	{
		# Following values will be replaced with _CGI_QUERY_VALUE.
			'realname'				=>	'No name',
			'email'					=>	undef,
			'subject'				=>	undef,
			'body'					=>	undef,
			'_Recipient'			=>	undef,
			'nexturl'				=>	undef,
			'config'				=>	undef,
			'referer'				=>	undef,
		},
		'_ParameterEvaluater'	=>	undef,
		'_CODE'					=>	'Shift_JIS',
		'_charset'				=>	'sjis',
		'_CGI_header'			=>	"",
		'InputFormTemplateFile'	=>	undef,
		'_HTMLTemplateFile'		=>	undef,
		'_TemplateData'	=>	undef,
		'_Replace_Key_in_Template'	=> {
			'_Aritcle_Body_'		=>	undef,
			'_Recipient_select_'	=>	undef,
			'_Hidden_parameter'		=>	undef,
			'_Form_Title_'			=> "Mail form",
			'_Action_Form_'			=> qq!<FORM ACTION="$s_cgi_uri" METHOD="POST">!,
			'_Document_Body_'		=>	undef,
			'_Input_Guide_'			=>	undef,
		},
		# These hidden paramater values must keep in _MailEntity.
		'_HiddenParameter_Key'	=>	['nexturl','config','referer'],
		'_Recipient_Alias'		=>	[],
		'_Recipient_Address'	=>	{},
		'_CharsetTable'			=>	{
			'ISO-2022-JP'			=>	'jis',
			'SHIFT_JIS'				=>	'sjis',
			'X-JIS'					=>	'sjis',
			'X-EUC-JP'				=>	'euc',
		},
		'_CodeTable'	=>	{
			'jis'					=>	'ISO-2022-JP',
			'sjis'					=>	'Shift_JIS',
		#	'sjis'					=>	'X-JIS',
			'euc'					=>	'X-EUC-JP',
		},
	}, $pkg;
	$rh_this->SelectAgentCODE();

#######################
# Defined CGI argument evaluation subroutine
#
	$rh_this->{'_ParameterEvaluater'} = {
#//////////////////////////////////////////
		'config' => sub {
		my($pkg,$string) = @_;
			$pkg->{'config'} = $string;
			$pkg->{'_MailEntity'}{'config'} = $string;
		},
#//////////////////////////////////////////
		'referer' => sub {
		my($pkg,$string) = @_;
			$pkg->{'_MailEntity'}{'referer'} = $string;
		},
#//////////////////////////////////////////
		'nexturl' => sub {
		my($pkg,$string) = @_;
			$pkg->{'_MailEntity'}{'nexturl'} = $string;
		},
#//////////////////////////////////////////
		'realname' => sub {
		my($pkg,$string) = @_;
			$pkg->{'_MailEntity'}{'realname'} = $string;
		},
#//////////////////////////////////////////
		'subject' => sub {
		my($pkg,$string) = @_;
			$pkg->{'_MailEntity'}{'subject'} = $string;
		},
#//////////////////////////////////////////
		'recipient' => sub {
		my($pkg,$string) = @_;
			$pkg->{'_MailEntity'}{'_Recipient'} = $string;
		},
#//////////////////////////////////////////
		'email' => sub {
		my($pkg) = shift;
			$pkg->{'_MailEntity'}{'email'} = shift;
			unless( $pkg->{'_MailEntity'}{'email'} ne '' )
			{
				$pkg->{'_AnyError'} |= 0x0100;
				$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} .= << "_EMPTY_EMAIL_FILED_";
<SPAN class="warning">Your E-mail</SPAN>: Your full E-mail address.<BR>
_EMPTY_EMAIL_FILED_
				return	0x0001;
			}
			unless( $pkg->{'_MailEntity'}{'email'} =~ m/[\w\-\.]+@[\w\-\.]+/ )
			{#E-mail addrress is invalid
				$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} .= << "_INVALID_EMAIL_ADDRESS_";
<SPAN class="warning">Your E-mail</SPAN>: Your E-mail address is invalid.<BR>
_INVALID_EMAIL_ADDRESS_
				return	0x0002;
			}
			return 0;
		},
#//////////////////////////////////////////
		'body' => sub {
		my($pkg) = shift;
		my($client_code);
			$pkg->{'_MailEntity'}{'body'} = shift;
			
			local($Mailfrom::body) = $pkg->{'_MailEntity'}{'body'};
			$client_code = &jcode::getcode(*Mailfrom::body);
			$pkg->SetClientCharset($client_code);
			return 0;
		},
	};
#######################

	return $rh_this;
}
#######################
# Method autoload
sub AUTOLOAD
{
my($pkg) = shift;
my($type) = ref($pkg) || croak "$pkg is not an object";
my($name) = $Mailform::AUTOLOAD;
	$name =~ s/.*://;
	unless( exists($pkg->{$name}) )
	{
#>>>DEBUG_CODE
		unless( $name eq 'debug' )
		{
#>>>DEBUG_CODE
			croak "Can't access `$name' field in object of class $type";
#>>>DEBUG_CODE
		}
		else
		{
			if( @_ )
			{
				foreach( @_ )
				{
					if( defined($pkg->{'_debug'}{$_}) )
					{
						delete $pkg->{'_debug'}{$_};
					}
					else
					{
						$pkg->{'_debug'}{$_} = 1;
					}
				}
			}
		}
		return wantarray ? keys(%{$pkg->{'_debug'}}) : $pkg->{'_debug'};
#>>>DEBUG_CODE
	}

	return $pkg->{$name} = shift if( @_ );
	return $pkg->{$name};
}
#	It's a scapegoat for AUTOLOAD.
sub	DESTROY{}
#######################
# Parse Query string
sub ParseQuery
{
my($pkg) = shift;
local($_);
my($key,$func,$value);
	$pkg->{'_CgiQuery'} = new CGI;

	foreach $key ( ($pkg->{'_CgiQuery'}->param) )
	{
		$value = $pkg->{'_CgiQuery'}->param($key);
#>>>DEBUG_CODE
		warn "$key=$value" if defined($pkg->{'_debug'}{'query_string'});
#>>>DEBUG_CODE
		if( defined($pkg->{'_ParameterEvaluater'}{$key}) )
		{
			$pkg->{'_AnyError'} |= &{$pkg->{'_ParameterEvaluater'}{$key}}($pkg,$value);
		}
		else
		{# It may be extended parameter. But do NOT override.
			$pkg->{$key} = $value unless exists($pkg->{$key});
		}
	}
	# Post mail. So checkout
	if( defined($pkg->{'func'}) && $pkg->{'func'} eq 'Send Mail' )
	{
		unless( defined($pkg->{'_MailEntity'}{'email'}) )
		{
			$pkg->{'_AnyError'} |= 0x0100;
			$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} .= << "_EMPTY_EMAIL_FILED_";
<SPAN class="warning">Your E-mail</SPAN>: Your full E-mail address.<BR>
_EMPTY_EMAIL_FILED_
		}
		unless( defined($pkg->{'_MailEntity'}{'subject'}) && $pkg->{'_MailEntity'}{'subject'} ne '' )
		{
			$pkg->{'_AnyError'} |= 0x0200;
			$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} .= << "_EMPTY_SUBJECT_FILED_";
<SPAN class="warning">Subject</SPAN>: subject of your item.<BR>
_EMPTY_SUBJECT_FILED_
		}
		unless( defined($pkg->{'_MailEntity'}{'body'}) && $pkg->{'_MailEntity'}{'body'} ne '' )
		{
			$pkg->{'_AnyError'} |= 0x0400;
			$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} .= << "_EMPTY_BODY_FILED_";
<SPAN class="warning">Message</SPAN>:Your .
_EMPTY_BODY_FILED_
		}
	}
}
#######################
# Set parameter
sub SetParameter
{
my($pkg) = shift @_;
my($rh_para) = shift @_;
my(%para_parser) = (
	'subject'	=>	[\&Set2ScalarRef,	\$pkg->{'_MailEntity'}{'subject'},	qq/Unknown "subject" argument type /],
	'nexturl'	=>	[\&Set2ScalarRef,	\$pkg->{'_MailEntity'}{'nexturl'},	qq/Unknown "nexturl" argument type /],
	'sendmail'	=>	[\&Set2ArrayRef,	\$pkg->{'_Config'}{'_Sendmail'},	qq/Unknown "sendmail" information type /],
	'redirect'	=>	[\&Set2ScalarRef,	\$pkg->{'_Config'}{'_Redirect'},	qq/Unknown "redirect" argument type /],
	);
	unless( ref($rh_para) )
	{# It's specify parameter file
		my($para) = new Params($rh_para);
		my($configname) = $pkg->config();
		warn "Handled as parameter file $rh_para." if defined($pkg->{'_debug'}{'configpara'});
		croak "Do not specify configuration name in CGI query string" unless defined($configname);
		$rh_para = $para->GetData($configname);

		$para->GetData('recipient', $pkg->{'_Recipient_Address'}, $pkg->{'_Recipient_Alias'});
	}
	else
	{
		$pkg->{'_Recipient_Address'} = $_[0];
		@{$pkg->{'_Recipient_Alias'}} = keys(%{$_[0]});
	}

	foreach( (keys(%para_parser)) )
	{
		if( defined( $rh_para->{$_} ) )
		{
			if( defined( ${$para_parser{$_}[1]} ) )
			{
#>>>DEBUG_CODE
				warn "Overrided paramater by query string : $_ " if defined($pkg->{'_debug'}{'configpara'});
#>>>DEBUG_CODE
				next;
			}
#>>>DEBUG_CODE
			warn "Parse parameter : $_ => ${$rh_para}{$_}" if defined($pkg->{'_debug'}{'configpara'});
#>>>DEBUG_CODE
			die "$para_parser{$_}[2] $pkg" unless &{$para_parser{$_}[0]}($para_parser{$_}[1],$rh_para->{$_});
		}
#>>>DEBUG_CODE
		else
		{
			warn "Undefined parameter : $_" if defined($pkg->{'_debug'}{'configpara'});
		}
#>>>DEBUG_CODE
	}
	if( $pkg->{'_MailEntity'}{'subject'} ne "" )
	{
		local($Mailform::subject) = $pkg->{'_MailEntity'}{'subject'};
		&jcode::convert(*Mailform::subject, $pkg->{'_charset'});
		$pkg->{'_MailEntity'}{'subject'} = $Mailform::subject;

	}
	$pkg->{'_MailEntity'}{'referer'} = $ENV{'HTTP_REFERER'} if(  defined( $ENV{'HTTP_REFERER'} ) );

}
#######################
sub OutResponse
{
my($pkg) = shift;
my($r_documentbody) = \$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'};
	unless( defined($pkg->{'func'}) && $pkg->{'func'} eq 'Send Mail' )
	{# Starting 
		$$r_documentbody .= << "_START_MESSAGE_";
<P>
The <STRONG>Your E-mail</STRONG> field needs to contain your mail address so replies go to the right place.<BR>
Type <STRONG>Your message</STRONG> into the text area below.<BR>
If the mail bounces for some reason, you will go back here.<BR>
</P>
_START_MESSAGE_
	}
	else
	{# Check CGI POST data
		if( $pkg->{'_AnyError'} )
		{
			$$r_documentbody =<< "_ANY_ERRORS_";
<P class="warning">You must input following fields correctly</P>
$$r_documentbody
_ANY_ERRORS_
		}
	}
	if( ( defined($pkg->{'func'}) && $pkg->{'func'} eq 'Send Mail' ) && !$pkg->{'_AnyError'} )
	{
		$pkg->SendmailResult();
	}
	else
	{
		$pkg->InputForm();
	}
}
#######################
# Set code of source article
sub SetClientCode
{
my($pkg,$code) = @_;
my($trcode) = $code;
	$trcode =~ tr/[a-z]/[A-Z]/;
	if( defined($pkg->{'_CharsetTable'}{$trcode}) )
	{
		$pkg->{'_CODE'} = $code;
		$pkg->{'_charset'} = $pkg->{'_CharsetTable'}{$trcode};
		return 1;
	}
	else
	{
		warn "Unknown code type $code";
		return 0;
	}
}
sub SetClientCharset
{
my($pkg,$code) = @_;
my($trcode) = $code;
	$trcode =~ tr/[A-Z]/[a-z]/;
	if( !defined($trcode) || $trcode eq '' )
	{
		return 1;
	}
	elsif( defined($pkg->{'_CodeTable'}{$trcode}) )
	{
		$pkg->{'_charset'} = $code;
		$pkg->{'_CODE'} = $pkg->{'_CodeTable'}{$trcode};
		return 1;
	}
	else
	{
		warn "Unknown code type $code";
		return 0;
	}
}
sub SelectAgentCODE
{
my($pkg) = @_;
	if( defined($ENV{'HTTP_USER_AGENT'}) )
	{
		my($agent) = $ENV{'HTTP_USER_AGENT'};
		if( $agent =~ m!(Win|Mac|OS/2)!i )
		{
			$pkg->SetClientCode('Shift_JIS');
		}
		else
		{
			$pkg->SetClientCode('X-EUC-JP');
		}
	}
}
#######################
# Output input form for follow article
sub InputForm
{
my($pkg,$article) = @_;

	$pkg->CreateSelectList();
	$pkg->CreateHiddenParameter();

	$pkg->{'_TemplateData'} = $pkg->GetTemplate();
	$pkg->{'_Replace_Key_in_Template'}{'_Form_Title_'} ='Send Mail Form';
	$pkg->{'_Replace_Key_in_Template'}{'_Document_Body_'} = $pkg->GetFormTemplate();
	# This is for Form extract
	$pkg->{'_TemplateData'} = $pkg->ExtendHTMLTemplate();
	# This is for replace keyword in form template
	$pkg->{'_TemplateData'} = $pkg->ExtendHTMLTemplate();
	$pkg->OutHTML();
	return;
}
#######################
# Note.
#	Can not see result page. Because redirected by location header.
#	So, custmizing is meaningless.
sub SendmailResult
{
my($pkg) = @_;

	$pkg->{'_TemplateData'} = $pkg->GetTemplate();
	$pkg->{'_Replace_Key_in_Template'}{'_Form_Title_'} ='Thank You For Filling Out This Form';
	$pkg->{'_MailEntity'}{'nexturl'} = &getURI( $pkg->{'_MailEntity'}{'referer'}, $pkg->{'_MailEntity'}{'nexturl'}) if defined($pkg->{'_MailEntity'}{'referer'});
#>>>DEBUG_CODE
	warn $pkg->{'_MailEntity'}{'referer'} if defined($pkg->{'_debug'}{'configpara'});
	warn $pkg->{'_MailEntity'}{'nexturl'} if defined($pkg->{'_debug'}{'configpara'});
#>>>DEBUG_CODE

	$pkg->{"_CGI_header"} .="Location: $pkg->{'_MailEntity'}{'nexturl'}";
	$pkg->{'_Replace_Key_in_Template'}{'_Document_Body_'} = << "_THANK_YOU_";
<P>Thank you for taking the time to fill out my feedback form. </P>
<HR>
<P>If do not redirect automaticaly, <A HREF="$pkg->{'_MailEntity'}{'nexturl'}">click here.</A></P>
_THANK_YOU_

	$pkg->{'_TemplateData'} = $pkg->ExtendHTMLTemplate();
	$pkg->OutHTML();
	$pkg->SendMail();
	return;
}
sub SendMail
{
my($pkg) = @_;
my($sendmail_command) = shift(@{$pkg->{'_Config'}{'_Sendmail'}});
local($Mailfrom::article);
my($mime_header);
my($indent);
my($clienthost);
my(@maillist) = $pkg->ExtractEmail($pkg->{'_MailEntity'}{'_Recipient'});
	$pkg->EncodeUnsafeCharacter( \$pkg->{'_MailEntity'}{'realname'} );
	$pkg->EncodeUnsafeCharacter(\$pkg->{'_MailEntity'}{'subject'});
	foreach( @{$pkg->{'_Config'}{'_Sendmail'}} )
	{
		die "Fail parameter extract in $_"
			unless( $sendmail_command =~ s/%s/$pkg->{'_MailEntity'}{$_}/e );
	}
	$Mailfrom::article = $pkg->{'_MailEntity'}{'body'};
	&jcode::convert(*Mailfrom::article, 'jis', $pkg->{'_charset'});	# convert to JIS code

	$indent = "To: ";
	
	die "No body recieve this mail!" if( @maillist == 0 );
	foreach( @maillist )
	{
		$mime_header .= "$indent$_";
		$indent = ",\n    ";
	}
	
	$mime_header .=  "\nX-User-Agent: $ENV{'HTTP_USER_AGENT'}" if( defined($ENV{'HTTP_USER_AGENT'}) );

	if( defined($pkg->{'_debug'}{'sendmail'}) )
	{
		die "Undefined redirect file for sendmail command test log." unless defined($pkg->{'_Config'}{'_Redirect'});
		open(MAIL, ">$pkg->{'_Config'}{'_Redirect'}") or die "Can't open $pkg->{'_Config'}{'_Redirect'}:$!";
		print MAIL "$sendmail_command\n";
	}
	else
	{
		open(MAIL, "| $sendmail_command")|| die "Can't open $sendmail_command:$!";
	}
	$clienthost = (gethostbyaddr(pack("C4", split(/\./, $ENV{'REMOTE_ADDR'})), 2))[0] if defined($ENV{'REMOTE_ADDR'});
	unless( defined($clienthost) && $clienthost ne '' )
	{
		$clienthost = defined($ENV{'REMOTE_HOST'}) ? $ENV{'REMOTE_HOST'}:$ENV{'REMOTE_ADDR'};
	}


	print MAIL <<"_END_OF_MAIL_";
$mime_header
From: $pkg->{'_MailEntity'}{'realname'} <$pkg->{'_MailEntity'}{'email'}>
Subject: $pkg->{'_MailEntity'}{'subject'}
X-Real-Host-From: $clienthost
MIME-Version: 1.0
Content-type: text/plain; charset=ISO-2022-JP
Content-Transfer-Encoding: 7bit

$Mailfrom::article

_END_OF_MAIL_

	close (MAIL);
	return;
}
########################
# Get template of default input form
sub GetFormTemplate
{
my($pkg) = shift;
my($template_data) = '';
	if( defined( $pkg->{'InputFormTemplateFile'} ) )
	{
		open( TMP, $pkg->{'InputFormTemplateFile'}) || die "Can not open $pkg->{'InputFormTemplateFile'} as template file:$!";
		while(<TMP>)
		{
			$template_data .= $_;
		}
	}
	else
	{
		$template_data = << "_MAIL_FORM_";
<!-- _Input_Guide_ -->
<!-- _Action_Form_ -->
<TABLE BORDER="0">
<TR>
	<TD>Your Name</TD><TD VALIGN="TOP">:</TD>
	<TD><INPUT TYPE="TEXT" MAXLENGTH="20" NAME="realname" _CGI_QUERY_VALUE realname></TD>
</TR>
<TR>
	<TD>Your E-mail</TD><TD VALIGN="TOP">:</TD>
	<TD><INPUT TYPE="TEXT" SIZE="40" MAXLENGTH="40" NAME="email" _CGI_QUERY_VALUE email></TD>
</TR>
<TR>
	<TD>Subject</TD><TD VALIGN="TOP">:</TD>
	<TD><INPUT TYPE="TEXT" SIZE="40" MAXLENGTH="60" NAME="subject" _CGI_QUERY_VALUE subject></TD>
</TR>
<TR>
	<!-- _Recipient_select_ -->
</TR>
<TR>
<TD></TD><TD VALIGN="TOP"></TD>
</TR>
</TABLE>

<P>
<INPUT TYPE="SUBMIT" NAME="func" VALUE="Send Mail">

<INPUT TYPE="RESET">
</P>
<P>
<TEXTAREA ROWS="30" COLS="70" NAME="body">
_CGI_QUERY body
</TEXTAREA>
</P>
<!-- _Hidden_parameter -->
</FORM>
_MAIL_FORM_
	}
	return $template_data;
}
#######################
# Set template file for input form
sub SetTemplate
{
my($pkg,$filename) = @_;
	$pkg->{'_HTMLTemplateFile'} = $filename;
}
########################
# Get template of HTML document
sub GetTemplate
{
my($pkg) = shift;
my($html_template);
	if( defined( $pkg->{'_HTMLTemplateFile'} ) )
	{
		open( TMP, $pkg->{'_HTMLTemplateFile'}) || die "Can not open $pkg->{'_HTMLTemplateFile'} as template file:$!";
		while(<TMP>)
		{
			$html_template .= $_;
		}
		close(TMP);
	}
	else
	{
		$html_template =<< "_DEFAULT_TEMPLATE_";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN">
<HTML>
<HEAD>
<META http-equiv="Content-Type" CONTENT="text/html; charset=$pkg->{'_CODE'}">
<META http-equiv="Content-Style-Type" content="text/css">
<TITLE><!-- _Form_Title_ --></TITLE>
<STYLE>
<!--
	.warning	{
		color: red;
		background-color: transparent;
		font-weight: bold;
	}
-->
</STYLE>
</HEAD>
<BODY>
<H1><!-- _Form_Title_ --></H1>
<HR>
<!-- _Document_Body_ -->
<HR>
</BODY>
</HTML>
_DEFAULT_TEMPLATE_
	}
	return $html_template;
}
###################
# Recipient address select form
sub CreateSelectList
{
my($pkg) = shift;
my($rh_relacekey) = \%{$pkg->{'_Replace_Key_in_Template'}};
my($work);
my($top_alias) = $pkg->{'_Recipient_Alias'}[0];
	die "Nobody recipient defined " if( @{$pkg->{'_Recipient_Alias'}} == 0 );
	if( @{$pkg->{'_Recipient_Alias'}} > 1 )
	{
		$rh_relacekey->{'_Recipient_select_'} = qq|<TD>Recipient</TD><TD VALIGN="TOP">:</TD>\n|;
		$rh_relacekey->{'_Recipient_select_'} .= qq|<TD><SELECT name="recipient" size="1">\n|;
		foreach( @{$pkg->{'_Recipient_Alias'}} )
		{
			$rh_relacekey->{'_Recipient_select_'} .= qq|<OPTION|;
			if( defined($pkg->{'_MailEntity'}{'_Recipient'}) && $_ eq $pkg->{'_MailEntity'}{'_Recipient'} )
			{
				$top_alias = $_;
				$rh_relacekey->{'_Recipient_select_'} .= qq| selected|;
			}
			$rh_relacekey->{'_Recipient_select_'} .= qq|>$_</OPTION>\n|;
		}
		$rh_relacekey->{'_Recipient_select_'} .= qq|</SELECT>\n</TD>\n|;
		$pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} .= "<P>This mail will send to $top_alias, if you do not change recipient.</P>";
	}
	else
	{
		$rh_relacekey->{'_Recipient_select_'} = qq|<TD></TD><TD VALIGN="TOP"></TD>\n|;
		$rh_relacekey->{'_Recipient_select_'} .= qq|<TD><INPUT TYPE="HIDDEN" NAME="recipient" VALUE="$top_alias"></TD>\n|;
	}
#>>>DEBUG_CODE
	warn $pkg->{'_Replace_Key_in_Template'}{'_Input_Guide_'} if defined($pkg->{'_debug'}{'template'});
#>>>DEBUG_CODE
}
sub CreateHiddenParameter
{
my($pkg) = @_;
	foreach( @{$pkg->{'_HiddenParameter_Key'}} )
	{
		$pkg->{'_Replace_Key_in_Template'}{'_Hidden_parameter'} .= qq|<INPUT type="hidden" name="$_" _CGI_QUERY_VALUE $_>\n|;
	}
#>>>DEBUG_CODE
	warn $pkg->{'_Replace_Key_in_Template'}{'_Hidden_parameter'} if defined($pkg->{'_debug'}{'template'});
#>>>DEBUG_CODE
}
#######################3
#
sub ExtendHTMLTemplate
{
my($pkg,$extend_source) = shift;
my($key_wordmatch);
	unless( defined($extend_source) )
	{
		$extend_source = $pkg->{'_TemplateData'};
	}
#>>>DEBUG_CODE
	warn $extend_source if defined($pkg->{'_debug'}{'template'});
#>>>DEBUG_CODE
	$key_wordmatch = '<!\-\-\s+(';
	$key_wordmatch .= join('|', keys(%{$pkg->{'_Replace_Key_in_Template'}}) );
	$key_wordmatch .= ')\s+\-\->';
#>>>DEBUG_CODE
	warn $key_wordmatch if defined($pkg->{'_debug'}{'template'});
#>>>DEBUG_CODE
	unless( $extend_source =~ s/$key_wordmatch/$pkg->GetKeyReplaceData($1)/eg )
	{
		warn "No match any _Replace_Key_in_Template";
	}
	return $extend_source;
}
sub GetKeyReplaceData
{
my($pkg,$keystring) = @_;
#>>>DEBUG_CODE
	warn "$keystring => $pkg->{'_Replace_Key_in_Template'}{$keystring}" if defined($pkg->{'_debug'}{'template'});
#>>>DEBUG_CODE
	if( defined($pkg->{'_Replace_Key_in_Template'}{$keystring}) )
	{
		return $pkg->{'_Replace_Key_in_Template'}{$keystring};
	}
	else
	{
		warn "Unknown replace key $keystring";
		return "<!-- $keystring -->";
	}
}
sub OutHTML
{
my($pkg) = @_;
	$pkg->ConvertTemplateCharset();
print <<"_HTTP_HEADER_";
Content-Type: text/html
$pkg->{"_CGI_header"}

_HTTP_HEADER_
	print $pkg->{'_TemplateData'};
}
######################################
# Convert Template code to client code
# Replace QUERY string in template
sub ConvertTemplateCharset
{
my($pkg,$template) = @_;
local($Mailform::template);

	unless( defined($template) )
	{
		$template = \$pkg->{'_TemplateData'};
	}

	$Mailform::template = $$template;
	&jcode::convert(*Mailform::template, $pkg->{'_charset'});	# convert to client code
	$$template = $Mailform::template;

	$$template =~ s/_CGI_QUERY_VALUE\s+([\w]+)\s*/{if(defined($pkg->{'_MailEntity'}{$1})){qq!VALUE="$pkg->{'_MailEntity'}{$1}"!;}else{"";}}/mge;
	$$template =~ s/_CGI_QUERY\s+([\w]+)\s*/{if(defined($pkg->{'_MailEntity'}{$1})){$pkg->{'_MailEntity'}{$1};}else{"";}}/mge;
}
###################
# Extract recipient mail address
#
sub ExtractEmail
{
my($pkg,$alias) = @_;
#>>>DEBUG_CODE
warn $alias	if defined($pkg->{'_debug'}{'alias'});
#>>>DEBUG_CODE
my(@mail) = split(/;/,$pkg->{'_Recipient_Address'}{$alias});
my(@maillist) = ();

	foreach( @mail )
	{
#>>>DEBUG_CODE
warn $_	if defined($pkg->{'_debug'}{'alias'});
#>>>DEBUG_CODE
		if( $_ =~ m/[\w\-\.]+@[\w\-\.]+/)
		{
			push(@maillist,$_);
		}
		else
		{#May be alias
			push(@maillist,$pkg->ExtractEmail($_));
		}
	}
	return @maillist;
}

###############################
# MIME header special character encode/decode
sub EncodeUnsafeCharacter
{
my($pkg,$r_string) = @_;
my($encoded);
my($dummy);
	if( $$r_string =~ m/^[\w\s]*$/ )
	{
		if( $$r_string =~ m/\s/ )
		{
			$$r_string = qq/"$$r_string"/;
		}
	}
	else
	{
		local($Mailform'string) = $$r_string;
		$dummy = &jcode::convert(*Mailform'string, 'jis', $pkg->{'_charset'});	# convert to JIS code
		$encoded = &encode_base64($Mailform'string);
		chop($encoded);
		$$r_string = "\=?ISO\-2022\-JP?B?$encoded?\=";
	}
}
sub DecodeUnsafeCharacter
{
my($r_string) = @_;
my($encoded);
	$r_string =~ s/\=\?ISO-2022-JP\?B\?([\w=]+)\?\=/&decode_base64($1)/ge;
	$r_string =~ s/\=([0-9A-Za-z]{2})/hex($1)/ge;
	return $r_string;
}
sub getURI
{
my($refer,$target) = @_;
my($scheme,$net_loc,$path);

	$refer =~ m|^(http)://([^/]*)/(.*)|i;
	$scheme = $1;
	$net_loc = $2;
	$path =$3;
	if( $target eq '' )
	{# Empty next URL
		$target = $refer;
	}
	elsif( $target =~ m!^(http|file)://([^/]*)/(.*)!i )
	{

	}
	elsif( $target =~ m!^/(.*)! )
	{# Is abs_path
		$target = "$scheme://$net_loc$target";
	}
	elsif( $target =~ m!\.\./! )
	{# Is backward move
		my(@a_refer) = split(/\//,$path);
		my(@a_target) = split(/\//,$target);
		unless( $refer =~ m!/$! )
		{# Skip object name
			pop(@a_refer);
		}
		while( $a_target[0] eq '..' )
		{
			pop(@a_refer);
			shift(@a_target);
		}
		$path = join('/',@a_refer);
		$target = join('/',@a_target);
		$target = "$scheme://$net_loc/$path/$target";
	}
	else
	{# Relative path
		$target = "$scheme://$net_loc/$path/$target";
	}
	return $target;
}
1;
